package
{
	import away3d.containers.ObjectContainer3D;
	import away3d.containers.View3D;
	import away3d.debug.AwayStats;
	import away3d.entities.Sprite3D;
	import away3d.filters.BloomFilter3D;
	import away3d.filters.BlurFilter3D;
	import away3d.filters.DepthOfFieldFilter3D;
	import away3d.lights.DirectionalLight;
	import away3d.lights.LightBase;
	import away3d.lights.PointLight;
	import away3d.materials.BitmapMaterial;
	import away3d.materials.ColorMaterial;
	import away3d.materials.methods.FogMethod;
	import away3d.primitives.Cube;
	import away3d.primitives.Plane;
	import away3d.primitives.Sphere;
	
	import flash.display.BitmapData;
	import flash.display.BlendMode;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.geom.ColorTransform;
	import flash.geom.Vector3D;
	import flash.utils.getTimer;
	
	/**
	 * Cubes01 Stage3D Demo by Felix Turner - www.airtight.cc
	 */
	
	[SWF( backgroundColor='0x333333', frameRate='60', width='800', height='600')]
	public class Cubes01 extends Sprite
	{
		private var CUBE_COUNT:int = 100;
		private var MATERIAL_COUNT:int = 30;
		private var CUBE_SIZE:int = 300;
		
		private var view : View3D;
		private var cubes:Array = [];
		private var cubeHolder : ObjectContainer3D;
		
		public function Cubes01(){
			super();
			
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			
			//init 3D world
			view = new View3D();
			view.camera.z = -1000;
			addChild(view);
			//init object to hold cubes and rotate
			cubeHolder = new ObjectContainer3D();
			view.scene.addChild(cubeHolder);
			
			view.addEventListener(Event.ADDED_TO_STAGE,onViewAdded);
			
			//add lights
			var light:PointLight = new PointLight();
			light.position = new Vector3D(-1000,1000,-1000);
			light.color = 0xffeeaa;
			view.scene.addChild(light);
			
			var light2:PointLight = new PointLight();
			light2.position = new Vector3D(1000,1000,1000);
			light2.color = 0xFFFFFF;
			view.scene.addChild(light2);
			
			//init materials
			var materials:Array = [];
			
			for (var i:int = 0 ; i < MATERIAL_COUNT; i ++ ){
				var material : ColorMaterial = new ColorMaterial(Math.random()*0xFFFFFF,.5);
				material.blendMode = BlendMode.ADD;
				material.lights = [light,light2];
				materials.push(material);
			}
			
			var spread:int = 1000;
			
			for (var j:int = 0 ; j < CUBE_COUNT; j ++ ){
				var s:Number = Math.random()*CUBE_SIZE +10;
				var cube:Cube = new Cube(materials[j % MATERIAL_COUNT], s,s,s);
				
				cubeHolder.addChild(cube);
				cubes.push(cube);
				
				cube.x = Math.random()* spread - spread/2;
				cube.y = Math.random()* spread - spread/2;
				cube.z = Math.random()* spread - spread/2;
				
				cube.rotationX = Math.random()*360;
				cube.rotationY = Math.random()*360;
				cube.rotationZ = Math.random()*360;
				
			}
			
			//add stats
			addChild(new AwayStats(view));
			
			this.addEventListener(Event.ENTER_FRAME, onEnterFrame);
			stage.addEventListener(Event.RESIZE, onStageResize);
			onStageResize(null);
			
		}
		
		private function onViewAdded(event : Event) : void{
			//improve performance by disabling error checking
			view.stage3DProxy.context3D.enableErrorChecking = false
		}
		
		private function onStageResize(event : Event) : void{
			view.width = stage.stageWidth;
			view.height = stage.stageHeight;
		}
		
		
		private function onEnterFrame(ev : Event) : void{
			cubeHolder.rotationY += .3;
			for (var i:int = 0 ; i < CUBE_COUNT; i ++ ){
				cubes[i].rotationX += .3;
			}
			view.render();
		}
	}
}